package org.processmining.models.connections.yawlnet;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.connections.annotations.ConnectionObjectFactory;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.Pair;
import org.yawlfoundation.yawl.editor.elements.model.YAWLFlowRelation;
import org.yawlfoundation.yawl.editor.elements.model.YAWLVertex;
import org.yawlfoundation.yawl.editor.net.NetGraph;

/**
 * Connection between Yawl net and log
 * @author David Piessens
 * @email d.a.m.piessens@student.tue.nl
 * @version May 29, 2010
 */
@Plugin(name = "Log yawl net connection factory", 
		parameterLabels = { "Log", "Event Classes", "Yawl net", "Relations" }, 
		returnTypes = YawlNetConnection.class, 
		returnLabels = "Log yawl net connection", 
		userAccessible = false)
@ConnectionObjectFactory
public class YawlNetConnection extends YawlLogModelConnection<YAWLVertex, YAWLFlowRelation> {
	/**
	 * Default constructor
	 * @param log
	 * @param classes
	 * @param graph
	 * @param relations
	 */
	public YawlNetConnection(XLog log, 
			XEventClasses classes,
			NetGraph graph,
			Set<YAWLVertex> vertices,
			Collection<Pair<YAWLVertex, XEventClass>> relations) {
		super(log, classes, graph, vertices, relations);
	}

	/**
	 * Static factory to produce LogFlexConnection
	 * @param context
	 * @param log
	 * @param classes
	 * @param graph
	 * @param relations
	 * @return
	 */
	@PluginVariant(requiredParameterLabels = { 0, 1, 2, 3 })
	public static YawlNetConnection logYawlConnectionFactory(PluginContext context, 
			XLog log, 
			XEventClasses classes,
			NetGraph graph,
			Object[] vertices,
			Collection<Pair<YAWLVertex, XEventClass>> relations) {
		// convert Object[] list of vertices to Collection<VertexContainer>
		Set<YAWLVertex> cVertices = new HashSet<YAWLVertex>(vertices.length);
		for (int i = 0; i > vertices.length; i++) {
			cVertices.add((YAWLVertex) vertices[i]);
		}
		YawlNetConnection logYawlConnection = new YawlNetConnection(log, classes, graph, cVertices, relations);
		return logYawlConnection;
	}
}
