package org.processmining.models.graphbased.directed.yawlperformancediagram;

import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.graphbased.directed.DirectedGraph;

/**
* @author David Piessens
* @email d.a.m.piessens@student.tue.nl
* @version May 29, 2010
*/
public interface YPD extends 
		DirectedGraph<YPDNode, YPDEdge<? extends YPDNode, ? extends YPDNode>> {
	
	/**
	 * Get label of an YPD
	 */
	public String getLabel();

	/**
	 * add a new YPD node with a certain label
	 * @param label
	 * @return
	 */
	public YPDTask addTask(String label);
	
	/**
	 * add a new YPD input condition with a certain label
	 * @param label
	 * @return
	 */
	public YPDInputCondition addInputCondition(String label);
	
	/**
	 * add a new YPD output condition with a certain label
	 * @param label
	 * @return
	 */
	public YPDOutputCondition addOutputCondition(String label);
	
	/**
	 * add a new YPD condition with a certain label
	 * @param label
	 * @return
	 */
	public YPDCondition addCondition(String label);

	/**
	 * Remove an YPDNode from an YPD
	 * @param ypdNodeElement
	 * @return
	 */
	public YPDNode removeNode(YPDNode ypdNodeElement);

	/**
	 * Get a set of nodes which refer to eventClass
	 * @param eventClass
	 * @return
	 */
	public Set<YPDNode> getNodes(XEventClass eventClass);
	
	// ARCS
	/**
	 * add new arc between source and target node
	 * @param source
	 * @param target
	 * @return
	 */
	public YPDEdge<YPDNode, YPDNode> addArc(YPDNode source, YPDNode target);

	/**
	 * remove an arc between source and target YPDNode
	 * @param source
	 * @param target
	 * @return
	 */
	public YPDEdge<YPDNode, YPDNode> removeArc(YPDNode source, YPDNode target);

	/**
	 * Get an arc between source and target node. Return null if there is no node
	 * @param source
	 * @param target
	 * @return
	 */
	public YPDEdge<YPDNode, YPDNode> getArc(YPDNode source, YPDNode target);
	
}
