package org.processmining.models.graphbased.directed.yawlperformancediagram;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;

import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.shapes.Decorated;
import org.processmining.models.shapes.Ellipse;

/**
* @author David Piessens
* @email d.a.m.piessens@student.tue.nl
* @version May 29, 2010
*/
public class YPDCondition extends YPDNode implements Decorated {

	private static final long serialVersionUID = -140549782392304694L;
	
	// related to cancellation events/regions
	private int timesCancelledByOtherTask = 0;
	private int timesCancelledByUser = 0;
	private boolean cancellation = false;
	
	public YPDCondition(AbstractDirectedGraph<YPDNode, YPDEdge<? extends YPDNode, ? extends YPDNode>> net, String label) {
		super(net, label);
		
		getAttributeMap().put(AttributeMap.LABEL, label);
		getAttributeMap().put(AttributeMap.SHAPE, new Ellipse());
		getAttributeMap().put(AttributeMap.SQUAREBB, true);
		getAttributeMap().put(AttributeMap.RESIZABLE, false);
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(40, 40));
		getAttributeMap().put(AttributeMap.SHOWLABEL, true);
	}

	public void decorate(Graphics2D g2d, double x, double y, double width, double height) {
		if (cancellation) {
			Color fillColor = Color.red;
		    String text = String.valueOf(getTimesCancelledByOtherTask());
		   // String textCancelledTask = nf.format(getTimesCancelledByOtherTask());
		   // String text = textCancelledUser + " / " + textCancelledTask;
		    getAttributeMap().put(AttributeMap.EDGECOLOR, fillColor);
		    getAttributeMap().put(AttributeMap.STROKECOLOR, fillColor);
		    String newLabel = ((String) getAttributeMap().get(AttributeMap.LABEL)) + " " + text;
			getAttributeMap().put(AttributeMap.LABEL, newLabel);
			
			// line added below to avoid multiple decorating
			this.cancellation = false;
		}
		
	}
	
	public void setTimesCancelledByOtherTask(int timesCancelledByOtherTask) {
		this.timesCancelledByOtherTask = timesCancelledByOtherTask;
	}

	public int getTimesCancelledByOtherTask() {
		return timesCancelledByOtherTask;
	}

	public void setTimesCancelledByUser(int timesCancelledByUser) {
		this.timesCancelledByUser = timesCancelledByUser;
	}

	public int getTimesCancelledByUser() {
		return timesCancelledByUser;
	}
	
	public void incrementCancelledByUser() {
		this.timesCancelledByUser++;
	}
	
	public void incrementCancelledByOtherTask() {
		this.timesCancelledByOtherTask++;
	}
	
	public void setCancellation(boolean cancellation) {
		this.cancellation = cancellation;
	}

	public boolean isCancellation() {
		return cancellation;
	}

}
