package org.processmining.models.graphbased.directed.yawlperformancediagram;

import java.util.Arrays;
import java.util.Collection;

import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.graphbased.directed.utils.GraphIterator;
import org.processmining.models.graphbased.directed.utils.GraphIterator.EdgeAcceptor;
import org.processmining.models.graphbased.directed.utils.GraphIterator.NodeAcceptor;

/**
* @author David Piessens
* @email d.a.m.piessens@student.tue.nl
* @version May 29, 2010
*/
public abstract class YPDNode extends AbstractDirectedGraphNode {

	private final AbstractDirectedGraph<YPDNode, YPDEdge<? extends YPDNode, ? extends YPDNode>> graph;

	public YPDNode(
			AbstractDirectedGraph<YPDNode, YPDEdge<? extends YPDNode, ? extends YPDNode>> net,
			String label) {
		super();
		this.graph = net;
	}

	/**
	 * return the graph which this node is belong to
	 */
	public AbstractDirectedGraph<YPDNode, YPDEdge<? extends YPDNode, ? extends YPDNode>> getGraph() {
		return graph;
	}
	
	/**
	 * get all successor for this node
	 * 
	 * @return
	 */
	public Collection<YPDNode> getVisibleSuccessors() {

		final NodeAcceptor<YPDNode> nodeAcceptor = new NodeAcceptor<YPDNode>() {
			public boolean acceptNode(YPDNode node, int depth) {
				return (depth != 0);
			}
		};

		Collection<YPDNode> nodes = GraphIterator.getDepthFirstSuccessors(this, getGraph(),
				new EdgeAcceptor<YPDNode, YPDEdge<? extends YPDNode, ? extends YPDNode>>() {

					public boolean acceptEdge(YPDEdge<? extends YPDNode, ? extends YPDNode> edge, int depth) {
						return !nodeAcceptor.acceptNode(edge.getSource(), depth);
					}
				}, nodeAcceptor);

		return Arrays.asList(nodes.toArray(new YPDNode[0]));
	}
	
	/**
	 * get all predecessor for this node
	 * 
	 * @return
	 */
	public Collection<YPDNode> getVisiblePredecessors() {
		final NodeAcceptor<YPDNode> nodeAcceptor = new NodeAcceptor<YPDNode>() {
			public boolean acceptNode(YPDNode node, int depth) {
				return (depth != 0);
			}
		};

		Collection<YPDNode> transitions = GraphIterator.getDepthFirstPredecessors(this, getGraph(),
				new EdgeAcceptor<YPDNode, YPDEdge<? extends YPDNode, ? extends YPDNode>>() {

					public boolean acceptEdge(YPDEdge<? extends YPDNode, ? extends YPDNode> edge, int depth) {
						return !nodeAcceptor.acceptNode(edge.getTarget(), depth);
					}
				}, nodeAcceptor);
		return Arrays.asList(transitions.toArray(new YPDNode[0]));
	}

}
